<?php namespace EwTools\Contenttypes;

use EwTools\Support\Collection;

/**
 * PropertyMapper
 *
 * @link    http://infomaker.se
 * @package EwTools\Contenttypes
 * @since   EwTools\Contenttypes\PropertyMapper 0.1
 */
class PropertyMapper {
    
    private $properties;
    
    public function __construct( $properties ) {
        $this->properties = $this->parseProperties( $properties );
    }
    
    public function getMap() {
        return $this->properties;
    }
    
    /**
     * Parse mapped properties into readable lists
     *
     * @since 0.1
     *
     * @param array $properties
     *
     * @return array
     */
    protected function parseProperties( array $properties = [] ) {
        return array_map( function ( $property_map ) {
            return is_array( $property_map ) ? array_map( [
                $this,
                'parseSelectors'
            ], $property_map ) : $this->parseSelectors( $property_map );
        }, $properties );
    }
    
    /**
     * Parse string of selectors into an array
     * Multiple selector may come separated by space, comma or pipe
     *
     * @param string $selectors
     *
     * @since 0.1
     * @return array
     */
    protected function parseSelectors( $selectors ) {
        return (array)preg_split( '/\s*(,|\|)\s*/', $selectors );
    }
    
    /**
     * Fetch a list of required properties based on what has been sent in during instantiation
     *
     * @since 0.1
     * @return array
     */
    public function requiredProperties() {
        return collect( $this->getMap() )->values()->flatten()->unique()->toArray();
    }
    
    /**
     *
     *
     * @param $object
     *
     * @since 0.1
     * @return array
     */
    public function extractProperties( $object ) {
        return array_map(function($value) use ($object){
            return $this->getPropertyValues( $value, $object );
        }, $this->getMap() );
    }
    
    /**
     * Fetch the data from the Object.
     * Support multi selectors for fallback support
     *
     * @param array $selectors
     * @param mixed $object
     *
     * @return array
     */
    protected function getPropertyValues( $selectors, $object ) {
        
        foreach ( (array)$selectors as $i ) {
            
            if( is_array( $i ) ) {
                return $this->getPropertyValues( $i, $object );
            }
            
            $value = $object->$i ?: [];
            
            if( ! empty( $value ) ) {
                return $value;
            }
        }
        
        return [];
    }
}